@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * callbacks.h
- * Copyright (C) John Stebbins 2008-2019 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2021 <stebbins@stebbins>
  *
  * callbacks.h is free software.
  *
@@ -41,7 +41,7 @@
                 g_thread_create((GThreadFunc)(f), (p), TRUE, NULL)
 #endif
 
-void ghb_check_all_depencencies(signal_user_data_t *ud);
+void ghb_check_all_dependencies(signal_user_data_t *ud);
 gboolean ghb_timer_cb(gpointer data);
 gboolean ghb_log_cb(GIOChannel *source, GIOCondition cond, gpointer data);
 void warn_log_handler(
