@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * preview.c
- * Copyright (C) John Stebbins 2008-2019 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2021 <stebbins@stebbins>
  *
  * preview.c is free software.
  *
@@ -111,13 +111,6 @@ ghb_par_scale(signal_user_data_t *ud, gint *width, gint *height, gint par_n, gin
         *height = *height * den / num;
 }
 
-void
-preview_set_size(signal_user_data_t *ud, int width, int height)
-{
-    ud->preview->width = width;
-    ud->preview->height = height;
-}
-
 void
 preview_set_render_size(signal_user_data_t *ud, int width, int height)
 {
@@ -126,6 +119,29 @@ preview_set_render_size(signal_user_data_t *ud, int width, int height)
     GhbSurface    * ss;
     GdkGeometry     geo;
 
+    if (ghb_dict_get_bool(ud->prefs, "reduce_hd_preview"))
+    {
+        gint factor = 80;
+        gint s_w, s_h;
+
+        ghb_monitor_get_size(GHB_WIDGET(ud->builder, "hb_window"), &s_w, &s_h);
+        if (s_w > 0 && s_h > 0)
+        {
+            int orig_w = width;
+            int orig_h = height;
+
+            if (width > s_w * factor / 100)
+            {
+                width = s_w * factor / 100;
+                height = height * width / orig_w;
+            }
+            if (height > s_h * factor / 100)
+            {
+                height = s_h * factor / 100;
+                width = orig_w * height / orig_h;
+            }
+        }
+    }
     widget = GHB_WIDGET (ud->builder, "preview_image");
     gtk_widget_set_size_request(widget, width, height);
     window = GTK_WINDOW(GHB_WIDGET(ud->builder, "preview_window"));
@@ -145,6 +161,25 @@ preview_set_render_size(signal_user_data_t *ud, int width, int height)
     ud->preview->render_height = height;
 }
 
+void
+preview_set_size(signal_user_data_t *ud, int width, int height)
+{
+    if (height == ud->preview->width &&
+        width == ud->preview->height)
+    {
+        // Rotation happened, fix up render size
+        preview_set_render_size(ud, ud->preview->render_height,
+                                ud->preview->render_width);
+    }
+    else if (width != ud->preview->width ||
+             height != ud->preview->height)
+    {
+        preview_set_render_size(ud, width, height);
+    }
+    ud->preview->width = width;
+    ud->preview->height = height;
+}
+
 void
 ghb_preview_init(signal_user_data_t *ud)
 {
@@ -933,20 +968,9 @@ void
 init_preview_image(signal_user_data_t *ud)
 {
     GtkWidget *widget;
-    gint width, height;
-
-    gint title_id, titleindex;
-    const hb_title_t *title;
 
     live_preview_stop(ud);
 
-    title_id = ghb_dict_get_int(ud->settings, "title");
-    title = ghb_lookup_title(title_id, &titleindex);
-    if (title == NULL && ud->preview->pix != NULL)
-    {
-        g_object_unref(ud->preview->pix);
-        ud->preview->pix = NULL;
-    }
     widget = GHB_WIDGET (ud->builder, "preview_frame");
     ud->preview->frame = ghb_widget_int(widget) - 1;
     if (ud->preview->encoded[ud->preview->frame])
@@ -971,8 +995,7 @@ init_preview_image(signal_user_data_t *ud)
     if (ud->preview->scaled_pix != NULL)
         g_object_unref(ud->preview->scaled_pix);
 
-    ud->preview->pix = ghb_get_preview_image(title, ud->preview->frame,
-                                             ud, &width, &height);
+    ud->preview->pix = ghb_get_preview_image(ud->preview->frame, ud);
     if (ud->preview->pix == NULL)
         return;
 
@@ -1404,7 +1427,7 @@ preview_state_cb(
         // Look for transition to iconified state.
         // Toggle "Show Preview" button when iconified.
         // I only do this because there seems to be no
-        // way to reliably disable the iconfy button without
+        // way to reliably disable the iconify button without
         // also disabling the maximize button.
         GdkEventWindowState * wse = (GdkEventWindowState*)event;
         if (wse->changed_mask & wse->new_window_state &
@@ -1464,13 +1487,17 @@ preview_resize_cb(
 G_MODULE_EXPORT void
 show_crop_changed_cb(GtkWidget *widget, signal_user_data_t *ud)
 {
+#if 0
+    // Disabled until we reimplement this or come up with something better
+    //
     g_debug("show_crop_changed_cb ()");
     ghb_widget_to_setting(ud->prefs, widget);
     ghb_check_dependency(ud, widget, NULL);
     ghb_live_reset(ud);
     if (gtk_widget_is_sensitive(widget))
-        ghb_set_scale(ud, GHB_PIC_KEEP_PAR);
+        ghb_set_scale(ud, 0);
     ghb_pref_save(ud->prefs, "preview_show_crop");
     ghb_rescale_preview_image(ud);
+#endif
 }
 
