@@ -1,6 +1,6 @@
 /* decssasub.c
 
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2021 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -25,14 +25,16 @@
  */
 
 #include "handbrake/handbrake.h"
+#include "handbrake/decavsub.h"
 #include "libavformat/avformat.h"
 
 struct hb_work_private_s
 {
-    AVFormatContext * ic;
-    AVPacket          avpkt;
-    hb_job_t        * job;
-    hb_subtitle_t   * subtitle;
+    AVFormatContext    * ic;
+    hb_avsub_context_t * ctx;
+    AVPacket           * pkt;
+    hb_job_t           * job;
+    hb_subtitle_t      * subtitle;
 
     // Time of first desired subtitle adjusted by reader_pts_offset
     uint64_t start_time;
@@ -46,6 +48,7 @@ static int extradataInit( hb_work_private_t * pv )
         hb_error("SSA demux found no streams");
         return 1;
     }
+
     AVStream * st = pv->ic->streams[0];
     if (st->codecpar->codec_id != AV_CODEC_ID_ASS)
     {
@@ -65,30 +68,47 @@ static int extradataInit( hb_work_private_t * pv )
 
 static int decssaInit( hb_work_object_t * w, hb_job_t * job )
 {
-    hb_work_private_t * pv;
+    hb_work_private_t * pv = NULL;
     int                 ii;
 
-    pv              = calloc( 1, sizeof( hb_work_private_t ) );
+    if (w->subtitle->config.src_filename == NULL)
+    {
+        hb_error("No SSA subtitle file specified");
+        goto fail;
+    }
+
+    pv = calloc( 1, sizeof( hb_work_private_t ) );
+    if (pv == NULL)
+    {
+        goto fail;
+    }
     w->private_data = pv;
     pv->job         = job;
     pv->subtitle    = w->subtitle;
 
-    if (pv->subtitle->config.src_filename != NULL)
+    pv->pkt = av_packet_alloc();
+    if (pv->pkt == NULL)
     {
-        if (avformat_open_input(&pv->ic, pv->subtitle->config.src_filename,
-            NULL, NULL ) < 0 )
-        {
-            hb_error("Could not open the SSA subtitle file '%s'\n",
-                     pv->subtitle->config.src_filename);
-            goto fail;
-        }
-        av_init_packet(&pv->avpkt);
+        hb_error("decssaInit: av_packet_alloc failed");
+        goto fail;
+    }
 
-        // Read SSA header and store in subtitle extradata
-        if (extradataInit(pv))
-        {
-            goto fail;
-        }
+    if (avformat_open_input(&pv->ic, pv->subtitle->config.src_filename,
+        NULL, NULL ) < 0 )
+    {
+        hb_error("Could not open the SSA subtitle file '%s'\n",
+                 pv->subtitle->config.src_filename);
+        goto fail;
+    }
+    pv->ctx = decavsubInit(w, job);
+    if (pv->ctx == NULL)
+    {
+        goto fail;
+    }
+
+    if (extradataInit(pv))
+    {
+        goto fail;
     }
 
     /*
@@ -132,10 +152,11 @@ static int decssaInit( hb_work_object_t * w, hb_job_t * job )
 fail:
     if (pv != NULL)
     {
-        if (pv->ic != NULL)
+        av_packet_free(&pv->pkt);
+        decavsubClose(pv->ctx);
+        if (pv->ic)
         {
             avformat_close_input(&pv->ic);
-            av_packet_unref(&pv->avpkt);
         }
         free(pv);
         w->private_data = NULL;
@@ -163,7 +184,7 @@ static hb_buffer_t * ssa_read( hb_work_private_t * pv )
         }
     }
 
-    if ((err = av_read_frame(pv->ic, &pv->avpkt)) < 0)
+    if ((err = av_read_frame(pv->ic, pv->pkt)) < 0)
     {
         if (err != AVERROR_EOF)
         {
@@ -171,35 +192,34 @@ static hb_buffer_t * ssa_read( hb_work_private_t * pv )
         }
         return hb_buffer_eof_init();
     }
-    AVStream * st = pv->ic->streams[pv->avpkt.stream_index];
+    AVStream * st = pv->ic->streams[pv->pkt->stream_index];
 
-    out = hb_buffer_init(pv->avpkt.size + 1);
-    memcpy(out->data, pv->avpkt.data, pv->avpkt.size);
-    out->data[pv->avpkt.size] = 0;
-    out->size = pv->avpkt.size;
+    out = hb_buffer_init(pv->pkt->size + 1);
+    memcpy(out->data, pv->pkt->data, pv->pkt->size);
+    out->data[pv->pkt->size] = 0;
+    out->size = pv->pkt->size;
 
     double tsconv = (double)90000. * st->time_base.num / st->time_base.den;
-    if (pv->avpkt.pts != AV_NOPTS_VALUE)
+    if (pv->pkt->pts != AV_NOPTS_VALUE)
     {
-        out->s.start = pv->avpkt.pts * tsconv +
+        out->s.start = pv->pkt->pts * tsconv +
                        pv->subtitle->config.offset * 90;
     }
-    if (pv->avpkt.dts != AV_NOPTS_VALUE)
+    if (pv->pkt->dts != AV_NOPTS_VALUE)
     {
-        out->s.renderOffset = pv->avpkt.dts * tsconv +
+        out->s.renderOffset = pv->pkt->dts * tsconv +
                               pv->subtitle->config.offset * 90;
     }
-    if (out->s.renderOffset != AV_NOPTS_VALUE && out->s.start == AV_NOPTS_VALUE)
+    if (out->s.renderOffset >= 0 && out->s.start == AV_NOPTS_VALUE)
     {
-       out->s.start = out->s.renderOffset;
+        out->s.start = out->s.renderOffset;
     }
-    else if (out->s.renderOffset == AV_NOPTS_VALUE &&
-             out->s.start != AV_NOPTS_VALUE)
+    else if (out->s.renderOffset == AV_NOPTS_VALUE && out->s.start >= 0)
     {
         out->s.renderOffset = out->s.start;
     }
-    int64_t pkt_duration = pv->avpkt.duration;
-    if (pkt_duration > 0)
+    int64_t pkt_duration = pv->pkt->duration;
+    if (pkt_duration != AV_NOPTS_VALUE)
     {
         out->s.duration = pkt_duration * tsconv;
         out->s.stop = out->s.start + out->s.duration;
@@ -209,7 +229,7 @@ static hb_buffer_t * ssa_read( hb_work_private_t * pv )
         out->s.duration = (int64_t)AV_NOPTS_VALUE;
     }
     out->s.type = SUBTITLE_BUF;
-    av_packet_unref(&pv->avpkt);
+    av_packet_unref(pv->pkt);
 
     if (out->s.stop  <= pv->start_time || out->s.start >= pv->stop_time)
     {
@@ -233,7 +253,6 @@ static hb_buffer_t * ssa_read( hb_work_private_t * pv )
         out->s.start -= pv->start_time;
         out->s.stop  -= pv->start_time;
     }
-
     return out;
 }
 
@@ -241,47 +260,34 @@ static int decssaWork( hb_work_object_t * w, hb_buffer_t ** buf_in,
                         hb_buffer_t ** buf_out )
 {
     hb_work_private_t * pv =  w->private_data;
-    hb_buffer_t       * in = *buf_in;
+    hb_buffer_t       * in;
+    int                 result;
 
-    *buf_in  = NULL;
-    *buf_out = NULL;
-    if (in == NULL && pv->ic != NULL)
+    in = ssa_read(pv);
+    if (in == NULL)
     {
-        in = ssa_read(pv);
-        if (in == NULL)
-        {
-            return HB_WORK_OK;
-        }
-    }
-    *buf_out = in;
-    if (in->s.flags & HB_BUF_FLAG_EOF)
-    {
-        return HB_WORK_DONE;
+        return HB_WORK_OK;
     }
 
-    // Not much to do here.  ffmpeg already supplies SSA subtitles in the
-    // required matroska packet format.
-    //
-    // We require string termination of the buffer
-    hb_buffer_realloc(in, ++in->size);
-    in->data[in->size - 1] = '\0';
-
-    return HB_WORK_OK;
+    result = decavsubWork(pv->ctx, &in, buf_out);
+    if (in != NULL)
+    {
+        hb_buffer_close(&in);
+    }
+    return result;
 }
 
 static void decssaClose( hb_work_object_t * w )
 {
-    hb_work_private_t * pv =  w->private_data;
+    hb_work_private_t * pv = w->private_data;
     if (pv != NULL)
     {
-        if (pv->ic != NULL)
-        {
-            avformat_close_input(&pv->ic);
-            av_packet_unref(&pv->avpkt);
-        }
+        decavsubClose(pv->ctx);
+        av_packet_free(&pv->pkt);
+        avformat_close_input(&pv->ic);
         free(pv);
-        w->private_data = NULL;
     }
+    w->private_data = NULL;
 }
 
 hb_work_object_t hb_decssasub =
