@@ -1,6 +1,6 @@
 /* internal.h
 
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2021 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -12,12 +12,7 @@
 
 #include "libavutil/imgutils.h"
 #include "libavutil/pixdesc.h"
-
 #include "handbrake/project.h"
-#if HB_PROJECT_FEATURE_QSV
-#include "handbrake/qsv_libav.h"
-#include "libavcodec/avcodec.h"
-#endif
 
 /***********************************************************************
  * common.c
@@ -64,6 +59,10 @@ void hb_job_setup_passes(hb_handle_t *h, hb_job_t *job, hb_list_t *list_pass);
  */
 typedef struct hb_buffer_s hb_buffer_t;
 
+#if HB_PROJECT_FEATURE_QSV
+#include "handbrake/qsv_libav.h"
+#endif
+
 struct hb_buffer_settings_s
 {
     enum { OTHER_BUF, AUDIO_BUF, VIDEO_BUF, SUBTITLE_BUF, FRAME_BUF } type;
@@ -154,10 +153,11 @@ struct hb_buffer_s
 #if HB_PROJECT_FEATURE_QSV
     struct qsv
     {
-        void           * qsv_atom;
-        AVFrame        * frame;
-        void           * filter_details;
-        hb_qsv_context * ctx;
+        void               * qsv_atom;
+        AVFrame            * frame;
+        void               * filter_details;
+        hb_qsv_context     * ctx;
+        HBQSVFramesContext * qsv_frames_ctx;
     } qsv_details;
 #endif
 
@@ -434,13 +434,10 @@ enum
     WORK_SYNC_VIDEO,
     WORK_SYNC_AUDIO,
     WORK_SYNC_SUBTITLE,
-    WORK_DECCC608,
     WORK_DECVOBSUB,
     WORK_DECSRTSUB,
-    WORK_DECUTF8SUB,
     WORK_DECTX3GSUB,
     WORK_DECSSASUB,
-    WORK_ENCVOBSUB,
     WORK_RENDER,
     WORK_ENCAVCODEC,
     WORK_ENCQSV,
@@ -457,7 +454,7 @@ enum
     WORK_ENCAVCODEC_AUDIO,
     WORK_MUX,
     WORK_READER,
-    WORK_DECPGSSUB
+    WORK_DECAVSUB
 };
 
 extern hb_filter_object_t hb_filter_detelecine;
