@@ -1,6 +1,6 @@
 /* qsv_common.h
  *
- * Copyright (c) 2003-2019 HandBrake Team
+ * Copyright (c) 2003-2021 HandBrake Team
  * This file is part of the HandBrake source code.
  * Homepage: <http://handbrake.fr/>.
  * It may be used under the terms of the GNU General Public License v2.
@@ -25,8 +25,8 @@ void hb_qsv_force_workarounds(); // for developers only
 
 #include "mfx/mfxvideo.h"
 #include "mfx/mfxplugin.h"
-#include "libavcodec/avcodec.h"
 #include "handbrake/hb_dict.h"
+#include "handbrake/qsv_libav.h"
 
 /* Minimum Intel Media SDK version (currently 1.3, for Sandy Bridge support) */
 #define HB_QSV_MINVERSION_MAJOR HB_QSV_MSDK_VERSION_MAJOR
@@ -44,7 +44,7 @@ void hb_qsv_force_workarounds(); // for developers only
 typedef struct hb_qsv_info_s
 {
     // each info struct only corresponds to one CodecId and implementation combo
-    const mfxU32  codec_id;
+    mfxU32  codec_id;
     mfxIMPL implementation;
 
     // whether the encoder is available for this implementation
@@ -74,20 +74,27 @@ typedef struct hb_qsv_info_s
 #define HB_QSV_CAP_OPTION2_IB_ADAPT  (1LL << 35)
 #define HB_QSV_CAP_OPTION2_LA_DOWNS  (1LL << 36)
 #define HB_QSV_CAP_OPTION2_NMPSLICE  (1LL << 37)
+#define HB_QSV_CAP_VPP_SCALING       (1LL << 38)
+#define HB_QSV_CAP_VPP_INTERPOLATION (1LL << 39)
 
     // TODO: add maximum encode resolution, etc.
 } hb_qsv_info_t;
 
 /* Intel Quick Sync Video utilities */
 hb_display_t * hb_qsv_display_init(void);
-int            hb_qsv_video_encoder_is_enabled(int encoder);
+int            hb_qsv_video_encoder_is_enabled(int adapter_index, int encoder);
 int            hb_qsv_audio_encoder_is_enabled(int encoder);
 int            hb_qsv_info_init();
 void           hb_qsv_info_print();
-hb_qsv_info_t* hb_qsv_info_get(int encoder);
+hb_list_t*     hb_qsv_adapters_list();
+hb_qsv_info_t* hb_qsv_encoder_info_get(int adapter_index, int encoder);
+int            hb_qsv_hardware_generation(int cpu_platform);
+int            hb_qsv_get_platform(int adapter_index);
+int            hb_qsv_get_adapter_index();
+int            hb_qsv_implementation_is_hardware(mfxIMPL implementation);
 
 /* Automatically load and unload any required MFX plug-ins */
-hb_list_t* hb_qsv_load_plugins  (hb_qsv_info_t *info, mfxSession session, mfxVersion version);
+hb_list_t* hb_qsv_load_plugins  (int adapter_index, hb_qsv_info_t *info, mfxSession session, mfxVersion version);
 void       hb_qsv_unload_plugins(hb_list_t     **_l,  mfxSession session, mfxVersion version);
 
 /* Intel Quick Sync Video DECODE utilities */
@@ -110,6 +117,25 @@ enum
     HB_QSV_PARAM_UNSUPPORTED,
 };
 
+/*
+ *  * Determine the "generation" of QSV hardware based on the CPU microarchitecture.
+ *   * Anything unknown is assumed to be more recent than the latest known generation.
+ *    * This avoids having to order the hb_cpu_platform enum depending on QSV hardware.
+ *     */
+enum
+{
+    QSV_G0, // third party hardware
+    QSV_G1, // Sandy Bridge or equivalent
+    QSV_G2, // Ivy Bridge or equivalent
+    QSV_G3, // Haswell or equivalent
+    QSV_G4, // Broadwell or equivalent
+    QSV_G5, // Skylake or equivalent
+    QSV_G6, // Kaby Lake or equivalent
+    QSV_G7, // Ice Lake or equivalent
+    QSV_G8, // Tiger Lake or equivalent
+    QSV_FU, // always last (future processors)
+};
+
 typedef struct
 {
     /*
@@ -174,11 +200,13 @@ int   hb_qsv_atobool (const char *str, int *err);
 int   hb_qsv_atoi    (const char *str, int *err);
 float hb_qsv_atof    (const char *str, int *err);
 
-int hb_qsv_param_default_preset(hb_qsv_param_t *param, mfxVideoParam *videoParam, hb_qsv_info_t *info, const char *preset);
-int hb_qsv_param_default       (hb_qsv_param_t *param, mfxVideoParam *videoParam, hb_qsv_info_t *info);
-int hb_qsv_param_parse         (hb_qsv_param_t *param,                            hb_qsv_info_t *info, const char *key, const char *value);
-int hb_qsv_profile_parse       (hb_qsv_param_t *param,                            hb_qsv_info_t *info, const char *profile_key, const int codec);
-int hb_qsv_level_parse         (hb_qsv_param_t *param,                            hb_qsv_info_t *info, const char *level_key);
+int hb_qsv_param_default_async_depth();
+int hb_qsv_param_default_preset     (hb_qsv_param_t *param, mfxVideoParam *videoParam, hb_qsv_info_t *info, const char *preset);
+int hb_qsv_param_default            (hb_qsv_param_t *param, mfxVideoParam *videoParam, hb_qsv_info_t *info);
+int hb_qsv_param_parse              (hb_qsv_param_t *param,                            hb_qsv_info_t *info, hb_job_t *job,  const char *key, const char *value);
+int hb_qsv_profile_parse            (hb_qsv_param_t *param,                            hb_qsv_info_t *info, const char *profile_key, const int codec);
+int hb_qsv_level_parse              (hb_qsv_param_t *param,                            hb_qsv_info_t *info, const char *level_key);
+int hb_qsv_param_parse_dx_index     (hb_job_t *job, const int dx_index);
 
 typedef struct
 {
@@ -200,62 +228,38 @@ uint8_t     hb_qsv_frametype_xlat(uint16_t qsv_frametype, uint16_t *out_flags);
 
 const char* hb_qsv_impl_get_name(int impl);
 const char* hb_qsv_impl_get_via_name(int impl);
-
-
-typedef struct QSVMid {
-    AVBufferRef *hw_frames_ref;
-    mfxHDL handle;
-
-    void *texture;
-
-    AVFrame *locked_frame;
-    AVFrame *hw_frame;
-    mfxFrameSurface1 surf;
-} QSVMid;
-
-typedef struct QSVFrame {
-    AVFrame *frame;
-    mfxFrameSurface1 surface;
-    mfxEncodeCtrl enc_ctrl;
-    mfxExtDecodedFrameInfo dec_info;
-    mfxExtBuffer *ext_param;
-
-    int queued;
-    int used;
-
-    struct QSVFrame *next;
-} QSVFrame;
-
-#define HB_POOL_SURFACE_SIZE (64)
-#define HB_POOL_ENCODER_SIZE (8)
-
-typedef struct EncQSVFramesContext {
-    AVBufferRef *hw_frames_ctx;
-    //void *logctx;
-
-    /* The memory ids for the external frames.
-     * Refcounted, since we need one reference owned by the QSVFramesContext
-     * (i.e. by the encoder/decoder) and another one given to the MFX session
-     * from the frame allocator. */
-    AVBufferRef *mids_buf;
-    QSVMid *mids;
-    int  nb_mids;
-    int pool[HB_POOL_SURFACE_SIZE];
-    void *input_texture;
-} EncQSVFramesContext;
+mfxIMPL     hb_qsv_dx_index_to_impl(int dx_index);
 
 /* Full QSV pipeline helpers */
+int hb_qsv_is_enabled(hb_job_t *job);
+hb_qsv_context* hb_qsv_context_init();
+void hb_qsv_context_uninit(hb_job_t *job);
+int hb_qsv_sanitize_filter_list(hb_job_t *job);
+int hb_qsv_hw_frames_init(AVCodecContext *s);
+int hb_create_ffmpeg_pool(hb_job_t *job, int coded_width, int coded_height, enum AVPixelFormat sw_pix_fmt, int pool_size, int extra_hw_frames, AVBufferRef **out_hw_frames_ctx);
+int hb_qsv_hw_filters_are_enabled(hb_job_t *job);
 int hb_qsv_full_path_is_enabled(hb_job_t *job);
 AVBufferRef *hb_qsv_create_mids(AVBufferRef *hw_frames_ref);
-hb_buffer_t* hb_qsv_copy_frame(AVFrame *frame, hb_qsv_context *qsv_ctx);
-void hb_qsv_get_free_surface_from_pool(const int start_index, const int end_index, QSVMid **out_mid, mfxFrameSurface1 **out_surface);
-int hb_qsv_replace_surface_mid(const QSVMid *mid, mfxFrameSurface1 *surface);
-int hb_qsv_release_surface_from_pool(const QSVMid *mid);
+int hb_qsv_attach_surface_to_video_buffer(hb_job_t *job, hb_buffer_t* buf, int is_vpp);
+int hb_qsv_copy_video_buffer_to_video_buffer(hb_job_t *job, hb_buffer_t* in, hb_buffer_t* out, int is_vpp);
+hb_buffer_t* hb_qsv_copy_avframe_to_video_buffer(hb_job_t *job, AVFrame *frame, int is_vpp);
+int hb_qsv_get_free_surface_from_pool(HBQSVFramesContext* hb_enc_qsv_frames_ctx, AVFrame* frame, QSVMid** out_mid);
+void hb_qsv_get_free_surface_from_pool_with_range(HBQSVFramesContext* hb_enc_qsv_frames_ctx, const int start_index, const int end_index, QSVMid** out_mid, mfxFrameSurface1** out_surface);
+int hb_qsv_get_mid_by_surface_from_pool(HBQSVFramesContext* hb_enc_qsv_frames_ctx, mfxFrameSurface1 *surface, QSVMid **out_mid);
+int hb_qsv_replace_surface_mid(HBQSVFramesContext* hb_qsv_frames_ctx, const QSVMid *mid, mfxFrameSurface1 *surface);
+int hb_qsv_release_surface_from_pool_by_surface_pointer(HBQSVFramesContext* hb_enc_qsv_frames_ctx, const mfxFrameSurface1 *surface);
 int hb_qsv_get_buffer(AVCodecContext *s, AVFrame *frame, int flags);
 enum AVPixelFormat hb_qsv_get_format(AVCodecContext *s, const enum AVPixelFormat *pix_fmts);
 int hb_qsv_preset_is_zero_copy_enabled(const hb_dict_t *job_dict);
 void hb_qsv_uninit_dec(AVCodecContext *s);
-void hb_qsv_uninit_enc();
+void hb_qsv_uninit_enc(hb_job_t *job);
+int hb_qsv_parse_adapter_index(hb_job_t *job);
+int hb_qsv_setup_job(hb_job_t *job);
+int hb_qsv_decode_h264_is_supported(int adapter_index);
+int hb_qsv_decode_h265_is_supported(int adapter_index);
+int hb_qsv_decode_h265_10_bit_is_supported(int adapter_index);
+int hb_qsv_decode_av1_is_supported(int adapter_index);
+int hb_qsv_decode_codec_supported_codec(int adapter_index, int video_codec_param, int pix_fmt);
 
 #endif // __LIBHB__
 #endif // HB_PROJECT_FEATURE_QSV
