@@ -1,6 +1,6 @@
 /* rendersub.c
 
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2021 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -22,7 +22,7 @@ struct hb_filter_private_s
     int                 sws_width;
     int                 sws_height;
 
-    // VOBSUB
+    // VOBSUB && PGSSUB
     hb_list_t         * sub_list; // List of active subs
 
     // SSA
@@ -188,13 +188,112 @@ static void blend( hb_buffer_t *dst, hb_buffer_t *src, int left, int top )
     }
 }
 
-// applies subtitle 'sub' YUVA420P buffer into destination 'buf'
-// 'buf' is currently YUV420P, but in future will be other formats as well
+static void blend8on1x( hb_buffer_t *dst, hb_buffer_t *src, int left, int top, int shift )
+{
+    int xx, yy;
+    int ww, hh;
+    int x0, y0;
+    int max;
+
+    uint8_t *y_in;
+    uint8_t *u_in;
+    uint8_t *v_in;
+    uint8_t *a_in;
+
+    uint16_t *y_out;
+    uint16_t *u_out;
+    uint16_t *v_out;
+    uint16_t alpha;
+
+    x0 = y0 = 0;
+    if( left < 0 )
+    {
+        x0 = -left;
+    }
+    if( top < 0 )
+    {
+        y0 = -top;
+    }
+
+    ww = src->f.width;
+    if( src->f.width - x0 > dst->f.width - left )
+    {
+        ww = dst->f.width - left + x0;
+    }
+    hh = src->f.height;
+    if( src->f.height - y0 > dst->f.height - top )
+    {
+        hh = dst->f.height - top + y0;
+    }
+
+    max = (256 << shift) -1;
+
+    // Blend luma
+    for( yy = y0; yy < hh; yy++ )
+    {
+        y_in   = src->plane[0].data + yy * src->plane[0].stride;
+        y_out   = (uint16_t*)(dst->plane[0].data + ( yy + top ) * dst->plane[0].stride);
+        a_in = src->plane[3].data + yy * src->plane[3].stride;
+        for( xx = x0; xx < ww; xx++ )
+        {
+            alpha = a_in[xx] << shift;
+            /*
+             * Merge the luminance and alpha with the picture
+             */
+            y_out[left + xx] =
+                ( (uint32_t)y_out[left + xx] * ( max - alpha ) +
+                     ((uint32_t)y_in[xx] << shift) * alpha ) / max;
+        }
+    }
+
+    // Blend U & V
+    int hshift = 0;
+    int wshift = 0;
+    if( dst->plane[1].height < dst->plane[0].height )
+        hshift = 1;
+    if( dst->plane[1].width < dst->plane[0].width )
+        wshift = 1;
+
+    for( yy = y0 >> hshift; yy < hh >> hshift; yy++ )
+    {
+        u_in = src->plane[1].data + yy * src->plane[1].stride;
+        u_out = (uint16_t*)(dst->plane[1].data + ( yy + ( top >> hshift ) ) * dst->plane[1].stride);
+        v_in = src->plane[2].data + yy * src->plane[2].stride;
+        v_out = (uint16_t*)(dst->plane[2].data + ( yy + ( top >> hshift ) ) * dst->plane[2].stride);
+        a_in = src->plane[3].data + ( yy << hshift ) * src->plane[3].stride;
+
+        for( xx = x0 >> wshift; xx < ww >> wshift; xx++ )
+        {
+            alpha = a_in[xx << wshift] << shift;
+
+            // Blend averge U and alpha
+            u_out[(left >> wshift) + xx] =
+                ( (uint32_t)u_out[(left >> wshift) + xx] * ( max - alpha ) +
+                  ((uint32_t)u_in[xx] << shift) * alpha ) / max;
+
+            // Blend V and alpha
+            v_out[(left >> wshift) + xx] =
+                ( (uint32_t)v_out[(left >> wshift) + xx] * ( max - alpha ) +
+                  ((uint32_t)v_in[xx] << shift) * alpha ) / max;
+        }
+    }
+}
+
 // Assumes that the input destination buffer has the same dimensions
 // as the original title dimensions
 static void ApplySub( hb_filter_private_t * pv, hb_buffer_t * buf, hb_buffer_t * sub )
 {
-    blend( buf, sub, sub->f.x, sub->f.y );
+    switch (pv->output.pix_fmt) {
+        case AV_PIX_FMT_YUV420P10:
+            blend8on1x(buf, sub, sub->f.x, sub->f.y, 2);
+            break;
+        case AV_PIX_FMT_YUV420P12:
+            blend8on1x(buf, sub, sub->f.x, sub->f.y, 4);
+            break;
+        default:
+            blend(buf, sub, sub->f.x, sub->f.y);
+            break;
+    }
 }
 
 static hb_buffer_t * ScaleSubtitle(hb_filter_private_t *pv,
@@ -249,8 +348,8 @@ static hb_buffer_t * ScaleSubtitle(hb_filter_private_t *pv,
             if (pv->sws!= NULL)
                 sws_freeContext(pv->sws);
             pv->sws = hb_sws_get_context(
-                                sub->f.width, sub->f.height, sub->f.fmt,
-                                scaled->f.width, scaled->f.height, sub->f.fmt,
+                                sub->f.width, sub->f.height, sub->f.fmt, AVCOL_RANGE_MPEG,
+                                scaled->f.width, scaled->f.height, sub->f.fmt, AVCOL_RANGE_MPEG,
                                 SWS_LANCZOS|SWS_ACCURATE_RND, SWS_CS_DEFAULT);
             pv->sws_width   = width;
             pv->sws_height  = height;
@@ -345,6 +444,8 @@ static void ApplyVOBSubs( hb_filter_private_t * pv, hb_buffer_t * buf )
     int ii;
     hb_buffer_t *sub, *next;
 
+    // Note that VOBSUBs can overlap in time.
+    // I.e. more than one may be rendered to the screen at once.
     for( ii = 0; ii < hb_list_count(pv->sub_list); )
     {
         sub = hb_list_item( pv->sub_list, ii );
@@ -577,7 +678,7 @@ static int ssa_post_init( hb_filter_object_t * filter, hb_job_t * job )
     }
 
     ass_set_use_margins( pv->renderer, 0 );
-    ass_set_hinting( pv->renderer, ASS_HINTING_LIGHT ); // VLC 1.0.4 uses this
+    ass_set_hinting( pv->renderer, ASS_HINTING_NONE );
     ass_set_font_scale( pv->renderer, 1.0 );
     ass_set_line_spacing( pv->renderer, 1.0 );
 
@@ -602,7 +703,7 @@ static int ssa_post_init( hb_filter_object_t * filter, hb_job_t * job )
     ass_set_frame_size( pv->renderer, width, height);
 
     double par = (double)job->par.num / job->par.den;
-    ass_set_aspect_ratio( pv->renderer, 1, par );
+    ass_set_pixel_aspect( pv->renderer, par );
 
     return 0;
 }
@@ -713,6 +814,7 @@ static void textsub_close( hb_filter_object_t * filter )
 static void process_sub(hb_filter_private_t *pv, hb_buffer_t *sub)
 {
     int64_t start, dur;
+    int size;
     char *ssa, *tmp;
 
     // libass expects every chunk to have a unique sequence number
@@ -729,7 +831,8 @@ static void process_sub(hb_filter_private_t *pv, hb_buffer_t *sub)
     // do not need to do special processing for stop == AV_NOPTS_VALUE
     start = sub->s.start;
     dur = sub->s.stop - sub->s.start;
-    ass_process_chunk(pv->ssaTrack, ssa, sub->size, start, dur);
+    size = strlen(ssa);
+    ass_process_chunk(pv->ssaTrack, ssa, size, start, dur);
     free(ssa);
 }
 
@@ -991,12 +1094,12 @@ static int hb_rendersub_post_init( hb_filter_object_t * filter, hb_job_t *job )
         case VOBSUB:
         {
             return vobsub_post_init( filter, job );
-        } break;
+        }
 
         case SSASUB:
         {
             return ssa_post_init( filter, job );
-        } break;
+        }
 
         case IMPORTSRT:
         case IMPORTSSA:
@@ -1004,25 +1107,25 @@ static int hb_rendersub_post_init( hb_filter_object_t * filter, hb_job_t *job )
         case TX3GSUB:
         {
             return textsub_post_init( filter, job );
-        } break;
+        }
 
         case CC608SUB:
         {
             return cc608sub_post_init( filter, job );
-        } break;
+        }
 
+        case DVBSUB:
         case PGSSUB:
         {
             return pgssub_post_init( filter, job );
-        } break;
+        }
 
         default:
         {
             hb_log("rendersub: unsupported subtitle format %d", pv->type );
             return 1;
-        } break;
+        }
     }
-    return 0;
 }
 
 static int hb_rendersub_work( hb_filter_object_t * filter,
@@ -1035,12 +1138,12 @@ static int hb_rendersub_work( hb_filter_object_t * filter,
         case VOBSUB:
         {
             return vobsub_work( filter, buf_in, buf_out );
-        } break;
+        }
 
         case SSASUB:
         {
             return ssa_work( filter, buf_in, buf_out );
-        } break;
+        }
 
         case IMPORTSRT:
         case IMPORTSSA:
@@ -1049,18 +1152,19 @@ static int hb_rendersub_work( hb_filter_object_t * filter,
         case TX3GSUB:
         {
             return textsub_work( filter, buf_in, buf_out );
-        } break;
+        }
 
+        case DVBSUB:
         case PGSSUB:
         {
             return pgssub_work( filter, buf_in, buf_out );
-        } break;
+        }
 
         default:
         {
             hb_error("rendersub: unsupported subtitle format %d", pv->type );
             return 1;
-        } break;
+        }
     }
 }
 
@@ -1093,6 +1197,7 @@ static void hb_rendersub_close( hb_filter_object_t * filter )
             textsub_close( filter );
         } break;
 
+        case DVBSUB:
         case PGSSUB:
         {
             pgssub_close( filter );
